@@ -25,8 +25,8 @@ class QiDataController:
         instrument_manager = InstrumentManager()
         instrument_manager.load(config_dir, EnumMarket.期货)
 
+        self.instrument_manager = instrument_manager
         self.__trading_day = qi_data_directory.trading_day
-        self.__instrument_manager = instrument_manager
 
         self.__future_tick_path = qi_data_directory.future_tick
         self.__future_tick_cache_path = qi_data_directory.future_tick_cache
@@ -43,7 +43,7 @@ class QiDataController:
             file_path = os.path.join(path, trading_date.strftime('%Y%m%d'))
             file_path = os.path.join(file_path, instrument_id.split('.')[0] + ".tk")
             if instrument_id.lower() != 'index':
-                instrument = self.__instrument_manager[instrument_id.split('.')[0]]
+                instrument = self.instrument_manager[instrument_id.split('.')[0]]
                 tick_stream = TickStream(market, instrument_id, instrument.exchange_id, file_path)
                 tick_stream.read_by_count(tick_series, 0, sys.maxsize)
         except Exception as e:
@@ -58,7 +58,7 @@ class QiDataController:
             file_path = os.path.join(path, trading_date.strftime('%Y%m%d'))
             file_path = os.path.join(file_path, instrument_id.split('.')[0] + ".tk")
             if instrument_id.lower() != 'index':
-                instrument = self.__instrument_manager[instrument_id.split('.')[0]]
+                instrument = self.instrument_manager[instrument_id.split('.')[0]]
                 tick_stream = TickStream(market, instrument_id, instrument.exchange_id, file_path)
                 tick_stream.read_by_count(tick_series, 0, sys.maxsize)
         except Exception as e:
@@ -137,7 +137,7 @@ class QiDataController:
                     file_path = os.path.join(path, date.strftime("%Y%m%D"))
                     file_path = os.path.join(file_path, instrument_id.split('.')[0] + ".tk")
                     if instrument_id.lower() != "index":
-                        instrument = self.__instrument_manager[instrument_id.split('.')[0]]
+                        instrument = self.instrument_manager[instrument_id.split('.')[0]]
                         tick_stream = TickStream(market, instrument_id, instrument.exchange_id, file_path)
                         tick_stream.read_by_time(all_ticks, begin_time, end_time)
 
@@ -201,7 +201,7 @@ class QiDataController:
         lst_trading_days = TradingDayHelper.get_trading_days(begin_trading_date, end_trading_date)
         if bar_type == EnumBarType.second:
             bar_provider = BarProvider()
-            bar_provider.create_bar_provider_by_period(self.__instrument_manager, instrument_id, begin_time, end_trading_date, interval, bar_type,
+            bar_provider.create_bar_provider_by_period(self.instrument_manager, instrument_id, begin_time, end_trading_date, interval, bar_type,
                                                        *instrument_ids)
             for trading_day in lst_trading_days:
                 tick_series = self.__load_history_tick_series(market, instrument_id, trading_day)
@@ -209,14 +209,14 @@ class QiDataController:
                     bar_provider.add_tick(tick)
         elif (bar_type == EnumBarType.minute) | (bar_type == EnumBarType.hour):
             bar_provider = BarProvider()
-            bar_provider.create_bar_provider_by_period(self.__instrument_manager, instrument_id, begin_time, end_trading_date, interval, bar_type,
+            bar_provider.create_bar_provider_by_period(self.instrument_manager, instrument_id, begin_time, end_trading_date, interval, bar_type,
                                                        *instrument_ids)
             bar_series_min = self.load_min_bar_series(market, instrument_id, begin_trading_date, end_trading_date)
             for bar in bar_series_min:
                 bar_provider.add_bar(bar)
         elif bar_type == EnumBarType.day:
             bar_provider = BarProvider()
-            bar_provider.create_bar_provider_by_period(self.__instrument_manager, instrument_id, begin_trading_date, end_trading_date, interval, bar_type,
+            bar_provider.create_bar_provider_by_period(self.instrument_manager, instrument_id, begin_trading_date, end_trading_date, interval, bar_type,
                                                        *instrument_ids)
             bar_series_day = self.load_day_bar_series(market, instrument_id, begin_trading_date, end_trading_date)
             for bar in bar_series_day:
@@ -226,7 +226,7 @@ class QiDataController:
 
         bar_series = bar_provider.bar_series
         bar_provider = BarProvider()
-        bar_provider.create_bar_provider_by_trading_day(self.__instrument_manager, instrument_id, self.__trading_day, interval, bar_type, *instrument_ids)
+        bar_provider.create_bar_provider_by_trading_day(self.instrument_manager, instrument_id, self.__trading_day, interval, bar_type, *instrument_ids)
         for bar in bar_series:
             bar_provider.bar_series.append(bar)
 
@@ -276,7 +276,7 @@ class QiDataController:
             end_trading_date = TradingDayHelper.get_pre_trading_day(last_trading_day)
         else:
             end_trading_date = last_trading_day
-        lst_date_time_slices_one_day = BaseBarHelper.create_day_date_time_slice(self.__instrument_manager, instrument_id, end_trading_date, interval, bar_type,
+        lst_date_time_slices_one_day = BaseBarHelper.create_day_date_time_slice(self.instrument_manager, instrument_id, end_trading_date, interval, bar_type,
                                                                                 *instrument_ids)
         if len(lst_date_time_slices_one_day) > 0:
             n_one_day_count = len(lst_date_time_slices_one_day)
@@ -292,7 +292,7 @@ class QiDataController:
         begin_trading_date = TradingDayHelper.get_pre_trading_day(end_trading_date, n_trading_days - 1)
         lst_trading_days = TradingDayHelper.get_trading_days(begin_trading_date, end_trading_date)
         bar_provider = BarProvider()
-        bar_provider.create_bar_provider_by_period(self.__instrument_manager, instrument_id, begin_trading_date, end_trading_date, interval, bar_type,
+        bar_provider.create_bar_provider_by_period(self.instrument_manager, instrument_id, begin_trading_date, end_trading_date, interval, bar_type,
                                                    *instrument_ids)
         if bar_provider.bar_type == EnumBarType.second:
             for trading_day in lst_trading_days:
@@ -319,20 +319,20 @@ class QiDataController:
                 trading_date = TradingDayHelper.get_pre_trading_day(trading_date)
                 if bar_provider.bar_type == EnumBarType.second:
                     bar_provider = BarProvider()
-                    bar_provider.create_bar_provider_by_trading_day(self.__instrument_manager, instrument_id, trading_date, interval, bar_type, *instrument_ids)
+                    bar_provider.create_bar_provider_by_trading_day(self.instrument_manager, instrument_id, trading_date, interval, bar_type, *instrument_ids)
                     tick_series = self.__load_history_tick_series(market, instrument_id, trading_date)
                     for tick in tick_series:
                         bar_provider.add_tick(tick)
                 elif (bar_provider.bar_type == EnumBarType.minute) | (bar_provider.bar_type == EnumBarType.hour):
                     bar_provider = BarProvider()
-                    bar_provider.create_bar_provider_by_trading_day(self.__instrument_manager, instrument_id, trading_date, interval, bar_type, *instrument_ids)
+                    bar_provider.create_bar_provider_by_trading_day(self.instrument_manager, instrument_id, trading_date, interval, bar_type, *instrument_ids)
                     bar_series_min = self.load_min_bar_series(market, instrument_id, trading_date)
                     for bar in bar_series_min:
                         bar_provider.add_bar(bar)
                 elif bar_provider.bar_type == EnumBarType.day:
                     begin_date = TradingDayHelper.get_pre_trading_day(trading_date, interval - 1)
                     bar_provider = BarProvider()
-                    bar_provider.create_bar_provider_by_trading_day(self.__instrument_manager, instrument_id, trading_date, interval, bar_type, *instrument_ids)
+                    bar_provider.create_bar_provider_by_trading_day(self.instrument_manager, instrument_id, trading_date, interval, bar_type, *instrument_ids)
                     bar_series_day = self.load_day_bar_series(market, instrument_id, begin_date, end_trading_date)
                     trading_date = begin_date
                     for bar in bar_series_day:
@@ -347,7 +347,7 @@ class QiDataController:
                     break
 
         bar_provider = BarProvider()
-        bar_provider.create_bar_provider_by_trading_day(self.__instrument_manager, instrument_id, self.__trading_day, interval, bar_type, *instrument_ids)
+        bar_provider.create_bar_provider_by_trading_day(self.instrument_manager, instrument_id, self.__trading_day, interval, bar_type, *instrument_ids)
         for bar in bar_series:
             bar_provider.bar_series.append(bar)
 
@@ -362,7 +362,7 @@ class QiDataController:
         return bar_provider.bar_series
 
     def __combine_today_tick(self, bar_provider, market, end_time):
-        date_time_slice = self.__instrument_manager.get_living_date_time(self.__trading_day, bar_provider.instrument_id)
+        date_time_slice = self.instrument_manager.get_living_date_time(self.__trading_day, bar_provider.instrument_id)
         if end_time > date_time_slice.begin_time:
             # 加载当日数据
             tick_series = self.__load_today_tick_series(market, bar_provider.instrument_id, self.__trading_day)
@@ -416,7 +416,7 @@ class QiDataController:
         end_trading_date = YfTimeHelper.get_trading_day(end_date)
         lst_trading_days = TradingDayHelper.get_trading_days(begin_trading_date, end_trading_date)
         bar_provider = BarProvider()
-        bar_provider.create_bar_provider_by_period(self.__instrument_manager, instrument_id, begin_trading_date, end_trading_date, interval, bar_type,
+        bar_provider.create_bar_provider_by_period(self.instrument_manager, instrument_id, begin_trading_date, end_trading_date, interval, bar_type,
                                                    *instrument_ids)
         if bar_type == EnumBarType.second:
             for trading_day in lst_trading_days:
@@ -436,14 +436,10 @@ class QiDataController:
 
         return bar_provider.bar_series
 
-    def load_bar_series_by_length(self, market, instrument_id, interval, bar_type, max_length, end_time, *instrument_ids):
+    def load_bar_series_by_length(self, market, instrument_id, interval, bar_type, max_length, end_date, *instrument_ids):
         # 这里为了提升取数据的速度,根据时间预计算根数,不足再补
-        last_trading_day = YfTimeHelper.get_trading_day(end_time)
-        if last_trading_day == self.__trading_day:
-            end_trading_date = TradingDayHelper.get_pre_trading_day(last_trading_day)
-        else:
-            end_trading_date = last_trading_day
-        lst_date_time_slices_one_day = BaseBarHelper.create_day_date_time_slice(self.__instrument_manager, instrument_id, end_trading_date, interval, bar_type,
+        end_trading_date = TradingDayHelper.get_last_trading_day(end_date)
+        lst_date_time_slices_one_day = BaseBarHelper.create_day_date_time_slice(self.instrument_manager, instrument_id, end_trading_date, interval, bar_type,
                                                                                 *instrument_ids)
         if len(lst_date_time_slices_one_day) > 0:
             n_one_day_count = len(lst_date_time_slices_one_day)
@@ -459,7 +455,7 @@ class QiDataController:
         begin_trading_date = TradingDayHelper.get_pre_trading_day(end_trading_date, n_trading_days - 1)
         lst_trading_days = TradingDayHelper.get_trading_days(begin_trading_date, end_trading_date)
         bar_provider = BarProvider()
-        bar_provider.create_bar_provider_by_period(self.__instrument_manager, instrument_id, begin_trading_date, end_trading_date, interval, bar_type,
+        bar_provider.create_bar_provider_by_period(self.instrument_manager, instrument_id, begin_trading_date, end_trading_date, interval, bar_type,
                                                    *instrument_ids)
         if bar_provider.bar_type == EnumBarType.second:
             for trading_day in lst_trading_days:
@@ -486,20 +482,20 @@ class QiDataController:
                 trading_date = TradingDayHelper.get_pre_trading_day(trading_date)
                 if bar_provider.bar_type == EnumBarType.second:
                     bar_provider = BarProvider()
-                    bar_provider.create_bar_provider_by_trading_day(self.__instrument_manager, instrument_id, trading_date, interval, bar_type, *instrument_ids)
+                    bar_provider.create_bar_provider_by_trading_day(self.instrument_manager, instrument_id, trading_date, interval, bar_type, *instrument_ids)
                     tick_series = self.__load_history_tick_series(market, instrument_id, trading_date)
                     for tick in tick_series:
                         bar_provider.add_tick(tick)
                 elif (bar_provider.bar_type == EnumBarType.minute) | (bar_provider.bar_type == EnumBarType.hour):
                     bar_provider = BarProvider()
-                    bar_provider.create_bar_provider_by_trading_day(self.__instrument_manager, instrument_id, trading_date, interval, bar_type, *instrument_ids)
+                    bar_provider.create_bar_provider_by_trading_day(self.instrument_manager, instrument_id, trading_date, interval, bar_type, *instrument_ids)
                     bar_series_min = self.load_min_bar_series(market, instrument_id, trading_date)
                     for bar in bar_series_min:
                         bar_provider.add_bar(bar)
                 elif bar_provider.bar_type == EnumBarType.day:
                     begin_date = TradingDayHelper.get_pre_trading_day(trading_date, interval - 1)
                     bar_provider = BarProvider()
-                    bar_provider.create_bar_provider_by_trading_day(self.__instrument_manager, instrument_id, trading_date, interval, bar_type, *instrument_ids)
+                    bar_provider.create_bar_provider_by_trading_day(self.instrument_manager, instrument_id, trading_date, interval, bar_type, *instrument_ids)
                     bar_series_day = self.load_day_bar_series(market, instrument_id, begin_date, end_trading_date)
                     trading_date = begin_date
                     for bar in bar_series_day:
@@ -514,7 +510,7 @@ class QiDataController:
                     break
 
         bar_provider = BarProvider()
-        bar_provider.create_bar_provider_by_trading_day(self.__instrument_manager, instrument_id, self.__trading_day, interval, bar_type, *instrument_ids)
+        bar_provider.create_bar_provider_by_trading_day(self.instrument_manager, instrument_id, self.__trading_day, interval, bar_type, *instrument_ids)
         for bar in bar_series:
             bar_provider.bar_series.append(bar)
 
@@ -523,3 +519,11 @@ class QiDataController:
             del bar_provider.bar_series[0:remove_length]
 
         return bar_provider.bar_series
+
+    def on_bar(self, bar):
+        if bar.instrument_id in self.__bar_series_map.keys():
+            bar_providers = self.__bar_series_map[bar.instrument_id]
+            for bar_provider in bar_providers:
+                bar_provider.add_bar(bar)
+        else:
+            return
